using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Transactions;
using System.Runtime.Serialization;

namespace WCFService
{
    [ServiceContract]
    interface IServiceContract
    {
        [OperationContract]
        int ShortOperation(int a, int b);

        [OperationContract]
        [FaultContract(typeof(TooLongOperationException))]
        string LongOperation(string a, string b);
    }
    [ServiceBehavior(ConcurrencyMode = ConcurrencyMode.Single,
                     InstanceContextMode = InstanceContextMode.Single)]
    class OpService : IServiceContract
    {
        public int ShortOperation(int a, int b)
        {
            return a - b;
        }

        [TransactionFlow(TransactionFlowOption.Allowed)]
        [OperationBehavior(TransactionAutoComplete=true,
                           TransactionScopeRequired=true)]
        public string LongOperation(string a, string b)
        {
            Transaction tx = Transaction.Current;
            Console.WriteLine("LocalID: {0},DistID: {1}, Time: {2}", 
                        tx.TransactionInformation.LocalIdentifier, 
                        tx.TransactionInformation.DistributedIdentifier, 
                        tx.TransactionInformation.CreationTime);
            System.Threading.Thread.Sleep(5000);
            TooLongOperationException t = new TooLongOperationException();
            t.Description = "Took too long";
            t.Operation = "LongOperation";
            throw new FaultException<TooLongOperationException>(t);
            
            return a + " " + b;
        }

    }

    class Program
    {
        static void Main(string[] args)
        {
            using (ServiceHost sh = new ServiceHost(typeof(OpService)))
            {
                sh.Open();
                Console.WriteLine("Service running...");
                Console.WriteLine("Listening on:");
                foreach (ServiceEndpoint se in sh.Description.Endpoints)
                {
                    Console.WriteLine(se.ListenUri.ToString());
                }
                Console.ReadLine();
                sh.Close();
            }
        }
    }

    [DataContract]
    class TooLongOperationException
    {
        string operation;
        string description;

        [DataMember]
        public string Operation
        {
            get { return operation; }
            set { operation = value; }
        }

        [DataMember]
        public string Description
        {
            get { return description; }
            set { description = value; }
        }
    }
}
